local AceConfig = LibStub("AceConfig-3.0")
local AceDBOptions = LibStub("AceDBOptions-3.0")
local ACD = LibStub("AceConfigDialog-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(PetTimeToLive.AppName)
local LDB = LibStub:GetLibrary("LibDataBroker-1.1", true)
local LDBIcon = LibStub("LibDBIcon-1.0", true)

local Icon = "Interface\\Icons\\INV_Battery_02"
local MinFontSize = 5
local MaxFontSize = 30

local _

local FontOutlines = {
    [""] = L["None"],
    ["OUTLINE"] = L["Normal"],
    ["THICKOUTLINE"] = L["Thick"],
}

local FrameStratas = {
    ["HIGH"] = L["High"],
    ["MEDIUM"] = L["Medium"],
    ["LOW"] = L["Low"],
}

local options = {
    type = "group",
    name = PetTimeToLive.AppName,
    handler = PetTimeToLive,
    get = "getOption",
    set = "setOption",
    args = {
        main = {
            type = 'group',
            childGroups = 'tab',
            inline = true,
            name = PetTimeToLive.AppName,
            handler = PetTimeToLive,
            get = "getOption",
            set = "setOption",
            order = 10,
            args = {
                locked = {
                    type = 'toggle',
                    name = L["Locked"],
                    desc = L["Lock/Unlock display frame"],
                    order = 110,
                },
                config = {
                    type = 'execute',
                    name = L["Configure"],
                    desc = L["Bring up GUI configure dialog"],
                    guiHidden = true,
                    order = 300,
                    func = function() PetTimeToLive:openConfigDialog() end,
                },

                color = {
                    type = 'color',
                    hasAlpha = true,
                    name = L["Color"],
                    set = "setColor",
                    get = "getColor",
                    order = 120,
                },
                warnColor = {
                    type = 'color',
                    hasAlpha = true,
                    name = L["Warning Color"],
                    set = "setColor",
                    get = "getColor",
                    order = 121,
                },

                warnThreshold = {
                    type = 'range',
                    name = L["Warning Threshold"],
                    min = 0,
                    max = 10,
                    step = 0.1,
                    order = 125,
                },
                warnSound = {
                    type = 'toggle',
                    -- width = 'full',
                    name = L["Warning Sound"],
                    desc = L["Play a sound if remaining time gets under Warning Threshold"],
                    order = 126,
                },
                warnSoundName = {
                    type = "select", dialogControl = 'LSM30_Sound',
                    disabled = function() return not PetTimeToLive.db.profile.warnSound end,
                    -- width = 'full',
                    name = L["Warning Sound Name"],
                    values = AceGUIWidgetLSMlists.sound,
                    order = 127,
                },

                font = {
                    type = "select", dialogControl = 'LSM30_Font',
                    name = L["Font"],
                    --desc = L["Font"],
                    values = AceGUIWidgetLSMlists.font,
                    order = 135,
                },
                fontSize = {
                    type = 'range',
                    name = L["Font size"],
                    --desc = L["Font size"],
                    min = MinFontSize,
                    max = MaxFontSize,
                    step = 1,
                    order = 140,
                },
                fontOutline = {
                    type = 'select',
                    name = L["Font outline"],
                    --desc = L["Font outline"],
                    values = FontOutlines,
                    order = 150,
                },
                strata = {
                    type = 'select',
                    name = L["Strata"],
                    desc = L["Frame strata"],
                    values = FrameStratas,
                    order = 155,
                },
                bg = {
                    type = "group",
                    name = L["Background Options"],
                    disabled = function() return not PetTimeToLive.db.profile.bgEnabled end,
                    guiInline = true,
                    order = 158,
                    args = {
                        bgEnabled = {
                            type = 'toggle',
                            order = 1,
                            name = L["Enabled"],
                            disabled = false,
                        },
                        frameWidth = {
                            type = 'range',
                            disabled = false,
                            name = L["Width"],
                            min = 20,
                            max = 256,
                            step = 1,
                            order = 5,
                        },
                        frameHeight = {
                            type = 'range',
                            disabled = false,
                            name = L["Height"],
                            min = 16,
                            max = 64,
                            step = 1,
                            order = 6,
                        },
                        bgTexture = {
                            type = "select", dialogControl = 'LSM30_Background',
                            order = 11,
                            name = L["Background Texture"],
                            desc = L["Texture to use for the frame's background"],
                            values = AceGUIWidgetLSMlists.background,
                        },
                        bgBorderTexture = {
                            type = "select", dialogControl = 'LSM30_Border',
                            order = 12,
                            name = L["Border Texture"],
                            desc = L["Texture to use for the frame's border"],
                            values = AceGUIWidgetLSMlists.border,
                        },
                        bgColor = {
                            type = "color",
                            order = 13,
                            name = L["Background Color"],
                            desc = L["Frame's background color"],
                            hasAlpha = true,
                            set = "setBGColor",
                            get = "getBGColor",
                        },
                        bgBorderColor = {
                            type = "color",
                            order = 14,
                            name = L["Border Color"],
                            desc = L["Frame's border color"],
                            hasAlpha = true,
                            set = "setBGColor",
                            get = "getBGColor",
                        },
                        bgTile = {
                            type = "toggle",
                            order = 2,
                            name = L["Tile Background"],
                            desc = L["Tile the background texture"],
                        },
                        bgTileSize = {
                            type = "range",
                            order = 16,
                            name = L["Background Tile Size"],
                            desc = L["The size used to tile the background texture"],
                            min = 16, max = 256, step = 1,
                            disabled = function() return not PetTimeToLive.db.profile.bgEnabled or not PetTimeToLive.db.profile.bgTile end,
                        },
                        bgEdgeSize = {
                            type = "range",
                            order = 17,
                            name = L["Border Thickness"],
                            desc = L["The thickness of the border"],
                            min = 1, max = 16, step = 1,
                        },
                    },
                },
            },
        },
    },
}

function PetTimeToLive:getOption(info)
    return self.db.profile[info[#info]]
end

function PetTimeToLive:setOption(info, value)
    self.db.profile[info[#info]] = value
    self:applySettings()
end

local function getColor(dbcolor)
    return dbcolor.r, dbcolor.g, dbcolor.b, dbcolor.a
end

local function setColor(dbcolor, r, g, b, a)
    dbcolor.r, dbcolor.g, dbcolor.b, dbcolor.a = r, g, b, a
end

function PetTimeToLive:getColor(info)
    local dbcolor = self.db.profile[info[#info]]
    return getColor(dbcolor)
end

function PetTimeToLive:setColor(info, r, g, b, a)
    local dbcolor = self.db.profile[info[#info]]
    setColor(dbcolor, r, g, b, a)
end

function PetTimeToLive:setBGColor(info, r, g, b, a)
    local db = self.db.profile
    local dbcolor = db[info[#info]]
    setColor(dbcolor, r, g, b, a)
    self.mainFrame:SetBackdropColor(db.bgColor.r, db.bgColor.g, db.bgColor.b, db.bgColor.a)
    self.mainFrame:SetBackdropBorderColor(db.bgBorderColor.r, db.bgBorderColor.g, db.bgBorderColor.b, db.bgBorderColor.a)
end

function PetTimeToLive:getBGColor(info)
    local dbcolor = self.db.profile[info[#info]]
    return getColor(dbcolor)
end

function PetTimeToLive:registerSubOptions(name, opts)
    local appName = self.AppName .. "." .. name
    AceConfig:RegisterOptionsTable(appName, opts)
    return ACD:AddToBlizOptions(appName, opts.name or name, self.AppName)
end

function PetTimeToLive:setupOptions()
    self:setupLDB()
    AceConfig:RegisterOptionsTable(self.AppName, options.args.main)
    self.opts = ACD:AddToBlizOptions(self.AppName, self.AppName)
    local profiles =  AceDBOptions:GetOptionsTable(self.db)
    LibStub("LibDualSpec-1.0"):EnhanceOptions(profiles, self.db)
    profiles.order = 900
    options.args.profiles = profiles
    self.profiles = self:registerSubOptions('profiles', profiles)
    AceConfig:RegisterOptionsTable(self.AppName .. '.Cmd', options, {"pttl", self.AppName:lower()})
end

function PetTimeToLive:setupLDB()
    if (not LDB) then return end
    local ldb = {
        type = "launcher",
        icon = Icon,
        OnClick = function(frame, button)
            if (button == "LeftButton") then
                self:toggleLocked()
            elseif (button == "RightButton") then
                self:openConfigDialog()
            end
        end,
        OnTooltipShow = function(tt)
            tt:AddLine(self.AppName)
            tt:AddLine(L["|cffeda55fLeft Click|r to lock/unlock frame"])
            tt:AddLine(L["|cffeda55fRight Click|r to open the configuration window"])
        end,
    }
    LDB:NewDataObject(self.AppName, ldb)
    if (not LDBIcon) then return end
    LDBIcon:Register(self.AppName, ldb, self.db.profile.minimap)
    options.args.main.args.minimap = {
        type = 'toggle',
        name = L["Hide minimap icon"],
        order = 111,
        get = function() return self.db.profile.minimap.hide end,
        set = function(info, value)
            if (value) then
                LDBIcon:Hide(self.AppName)
            else
                LDBIcon:Show(self.AppName)
            end
            self.db.profile.minimap.hide = value
        end,
    }
end

function PetTimeToLive:openConfigDialog()
    InterfaceOptionsFrame_OpenToCategory(self.profiles) -- to expand our tree
    InterfaceOptionsFrame_OpenToCategory(self.opts)
end

